using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace QuickReminder
{
	public partial class ReminderForm : Form
	{
		#region Constants & enumerations

		public enum AlarmStates { Pending, Alarming, Off }
		private int alarmAnimationFrame = 0;

		#endregion

		#region Instance variables

		private AlarmStates alarmState;
		private Reminder reminder;

		#endregion

		#region Construction

		public ReminderForm()
		{
			InitializeComponent();
			Icon = global::QuickReminder.Properties.Resources.Reminder;
			AlarmState = AlarmStates.Off;
		}

		#endregion

		#region Properties

		public AlarmStates AlarmState
		{
			get { return alarmState; }
			set {
				alarmState = value;
				switch (alarmState) {
					case AlarmStates.Alarming: {
						timer.Interval = 250;
						timer.Enabled = true;
						notifyIcon.ShowBalloonTip(60 * 60 * 1000, reminder.NextOccurance.ToString(), reminder.Name, ToolTipIcon.None);
						break;
					}
					case AlarmStates.Pending: {
						Icon = global::QuickReminder.Properties.Resources.Reminder;
						timer.Interval = 1000;
						timer.Enabled = true;
						break;
					}
					case AlarmStates.Off: {
						Icon = global::QuickReminder.Properties.Resources.Reminder;
						timer.Enabled = false;
						if (reminder != null)
							reminder.Active = false;
						break;
					}
				}
			}
		}

		#endregion

		#region Event handlers

		protected override void OnVisibleChanged(EventArgs e)
		{
			base.OnVisibleChanged(e);

			if (reminder != null)
				ReminderToUserInterface();

			nameCombo.Items.Clear();
			foreach (Reminder availableReminder in MainForm.AllReminders)
				nameCombo.Items.Add(availableReminder);
			nameCombo.SelectedItem = reminder;

			ReflectInterface(this, e);
		}

		private void ReflectInterface(object sender, EventArgs e)
		{
			atTimePicker.Visible = atRadio.Checked;
			inMinutesNud.Visible = inRadio.Checked;
			minutesLabel.Visible = inRadio.Checked;
		}

		private void NotifyIcon_Click(object sender, EventArgs e)
		{
			Show();
		}

		private void Timer_Tick(object sender, EventArgs e)
		{
			if ((alarmState == AlarmStates.Pending) && (DateTime.Now >= reminder.NextOccurance))
				AlarmState = AlarmStates.Alarming;

			if (alarmState == AlarmStates.Alarming) {
				switch (alarmAnimationFrame++) {
					case 0: {
							notifyIcon.Icon = global::QuickReminder.Properties.Resources.Alarm1;
							break;
						}
					case 1: {
							notifyIcon.Icon = global::QuickReminder.Properties.Resources.Reminder;
							break;
						}
					case 2: {
							notifyIcon.Icon = global::QuickReminder.Properties.Resources.Alarm2;
							alarmAnimationFrame = 0;
							break;
						}
				}
			}
		}

		private void ClearButton_Click(object sender, EventArgs e)
		{
			AlarmState = AlarmStates.Off;
			Close();
		}

		private void SetButton_Click(object sender, EventArgs e)
		{
			if (String.IsNullOrEmpty(nameCombo.Text)) {
				MessageBox.Show("Name can not be blank");
				return;
			}

			if (reminder == null) {
				reminder = new Reminder();
				MainForm.AllReminders.Add(reminder);
			}

			reminder.Name = nameCombo.Text;
			reminder.ReminderType = (atRadio.Checked) ? ReminderTypes.Time : ReminderTypes.Interval;
			reminder.Time = (reminder.ReminderType == ReminderTypes.Time) ? TimeSpan.Parse(atTimePicker.Text) : new TimeSpan(0, (int) inMinutesNud.Value, 0);

			notifyIcon.Text = string.Format("{0} at {1}", reminder.Name, reminder.NextOccurance);
			timer.Enabled = true;

			AlarmState = AlarmStates.Pending;

			Hide();
		}

		private void NameCombo_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (nameCombo.SelectedItem != null) {
				Reminder potentialReminder = nameCombo.SelectedItem as Reminder;
				if (potentialReminder.Active) {
					reminder = potentialReminder.Clone();
					
					MainForm.AllReminders.Add(reminder);
				}
				else
					reminder = potentialReminder;

				ReminderToUserInterface();
			}
		}

		private void ReminderToUserInterface()
		{
			nameCombo.Text = reminder.Name;
			atRadio.Checked = (reminder.ReminderType == ReminderTypes.Time);
			inRadio.Checked = (reminder.ReminderType == ReminderTypes.Interval);
			atTimePicker.Value = (reminder.ReminderType == ReminderTypes.Time) ? reminder.NextOccurance : DateTime.Now;
			inMinutesNud.Value = (reminder.ReminderType == ReminderTypes.Interval) ? reminder.Time.Minutes : 0;
		}

		private void HideButton_Click(object sender, EventArgs e)
		{
			Hide();
		}

		#endregion
	}
}